@@ -30,11 +30,6 @@ _default = None
 # magic gettext number to separate context from message
 CONTEXT_SEPARATOR = "\x04"
 
-# Maximum number of characters that will be parsed from the Accept-Language
-# header to prevent possible denial of service or memory exhaustion attacks.
-# About 10x longer than the longest value shown on MDN’s Accept-Language page.
-ACCEPT_LANGUAGE_HEADER_MAX_LENGTH = 500
-
 # Format of Accept-Language header values. From RFC 2616, section 14.4 and 3.9
 # and RFC 3066, section 2.1
 accept_language_re = _lazy_re_compile(
@@ -42,10 +37,10 @@ accept_language_re = _lazy_re_compile(
         # "en", "en-au", "x-y-z", "es-419", "*"
         ([A-Za-z]{1,8}(?:-[A-Za-z0-9]{1,8})*|\*)
         # Optional "q=1.00", "q=0.8"
-        (?:\s*;\s*q=(0(?:\.\d{,3})?|1(?:\.0{,3})?))?
+        (?:\s*;\s*q=(0(?:\.[0-9]{,3})?|1(?:\.0{,3})?))?
         # Multiple accepts per header.
         (?:\s*,\s*|$)
-        """,
+    """,
     re.VERBOSE,
 )
 
@@ -53,16 +48,16 @@ language_code_re = _lazy_re_compile(
     r"^[a-z]{1,8}(?:-[a-z0-9]{1,8})*(?:@[a-z0-9]{1,20})?$", re.IGNORECASE
 )
 
-language_code_prefix_re = _lazy_re_compile(r"^/(\w+([@-]\w+)?)(/|$)")
+language_code_prefix_re = _lazy_re_compile(r"^/(\w+([@-]\w+){0,2})(/|$)")
 
 
 @receiver(setting_changed)
-def reset_cache(**kwargs):
+def reset_cache(*, setting, **kwargs):
     """
     Reset global state when LANGUAGES setting has been changed, as some
     languages should no longer be accepted.
     """
-    if kwargs["setting"] in ("LANGUAGES", "LANGUAGE_CODE"):
+    if setting in ("LANGUAGES", "LANGUAGE_CODE"):
         check_for_language.cache_clear()
         get_languages.cache_clear()
         get_supported_language_variant.cache_clear()
@@ -212,7 +207,7 @@ class DjangoTranslation(gettext_module.GNUTranslations):
     def _add_installed_apps_translations(self):
         """Merge translations from each installed app."""
         try:
-            app_configs = reversed(list(apps.get_app_configs()))
+            app_configs = reversed(apps.get_app_configs())
         except AppRegistryNotReady:
             raise AppRegistryNotReady(
                 "The translation infrastructure cannot be initialized before the "
@@ -479,12 +474,13 @@ def check_for_language(lang_code):
     )
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def get_languages():
     """
     Cache of settings.LANGUAGES in a dictionary for easy lookups by key.
+    Convert keys to lowercase as they should be treated as case-insensitive.
     """
-    return dict(settings.LANGUAGES)
+    return {key.lower(): value for key, value in dict(settings.LANGUAGES).items()}
 
 
 @functools.lru_cache(maxsize=1000)
@@ -515,7 +511,7 @@ def get_supported_language_variant(lang_code, strict=False):
         supported_lang_codes = get_languages()
 
         for code in possible_lang_codes:
-            if code in supported_lang_codes and check_for_language(code):
+            if code.lower() in supported_lang_codes and check_for_language(code):
                 return code
         if not strict:
             # if fr-fr is not supported, try fr-ca.
@@ -590,7 +586,7 @@ def get_language_from_request(request, check_path=False):
 
 
 @functools.lru_cache(maxsize=1000)
-def _parse_accept_lang_header(lang_string):
+def parse_accept_lang_header(lang_string):
     """
     Parse the lang_string, which is the body of an HTTP Accept-Language
     header, and return a tuple of (lang, q-value), ordered by 'q' values.
@@ -612,27 +608,3 @@ def _parse_accept_lang_header(lang_string):
         result.append((lang, priority))
     result.sort(key=lambda k: k[1], reverse=True)
     return tuple(result)
-
-
-def parse_accept_lang_header(lang_string):
-    """
-    Parse the value of the Accept-Language header up to a maximum length.
-
-    The value of the header is truncated to a maximum length to avoid potential
-    denial of service and memory exhaustion attacks. Excessive memory could be
-    used if the raw value is very large as it would be cached due to the use of
-    functools.lru_cache() to avoid repetitive parsing of common header values.
-    """
-    # If the header value doesn't exceed the maximum allowed length, parse it.
-    if len(lang_string) <= ACCEPT_LANGUAGE_HEADER_MAX_LENGTH:
-        return _parse_accept_lang_header(lang_string)
-
-    # If there is at least one comma in the value, parse up to the last comma
-    # before the max length, skipping any truncated parts at the end of the
-    # header value.
-    if (index := lang_string.rfind(",", 0, ACCEPT_LANGUAGE_HEADER_MAX_LENGTH)) > 0:
-        return _parse_accept_lang_header(lang_string[:index])
-
-    # Don't attempt to parse if there is only one language-range value which is
-    # longer than the maximum allowed length and so truncated.
-    return ()
