@@ -45,6 +45,7 @@ from .admin import (
     InvitationAdmin,
     NoListDisplayLinksParentAdmin,
     ParentAdmin,
+    ParentAdminTwoSearchFields,
     QuartetAdmin,
     SwallowAdmin,
 )
@@ -189,6 +190,42 @@ class ChangeListTests(TestCase):
         cl = ia.get_changelist_instance(request)
         self.assertEqual(cl.queryset.query.select_related, {"player": {}, "band": {}})
 
+    def test_many_search_terms(self):
+        parent = Parent.objects.create(name="Mary")
+        Child.objects.create(parent=parent, name="Danielle")
+        Child.objects.create(parent=parent, name="Daniel")
+
+        m = ParentAdmin(Parent, custom_site)
+        request = self.factory.get("/parent/", data={SEARCH_VAR: "daniel " * 80})
+        request.user = self.superuser
+
+        cl = m.get_changelist_instance(request)
+        with CaptureQueriesContext(connection) as context:
+            object_count = cl.queryset.count()
+        self.assertEqual(object_count, 1)
+        self.assertEqual(context.captured_queries[0]["sql"].count("JOIN"), 1)
+
+    def test_related_field_multiple_search_terms(self):
+        """
+        Searches over multi-valued relationships return rows from related
+        models only when all searched fields match that row.
+        """
+        parent = Parent.objects.create(name="Mary")
+        Child.objects.create(parent=parent, name="Danielle", age=18)
+        Child.objects.create(parent=parent, name="Daniel", age=19)
+
+        m = ParentAdminTwoSearchFields(Parent, custom_site)
+
+        request = self.factory.get("/parent/", data={SEARCH_VAR: "danielle 19"})
+        request.user = self.superuser
+        cl = m.get_changelist_instance(request)
+        self.assertEqual(cl.queryset.count(), 0)
+
+        request = self.factory.get("/parent/", data={SEARCH_VAR: "daniel 19"})
+        request.user = self.superuser
+        cl = m.get_changelist_instance(request)
+        self.assertEqual(cl.queryset.count(), 1)
+
     def test_result_list_empty_changelist_value(self):
         """
         Regression test for #14982: EMPTY_CHANGELIST_VALUE should be honored
@@ -583,6 +620,61 @@ class ChangeListTests(TestCase):
         cl.queryset.delete()
         self.assertEqual(cl.queryset.count(), 0)
 
+    def test_multiple_search_fields(self):
+        """
+        All rows containing each of the searched words are returned, where each
+        word must be in one of search_fields.
+        """
+        band_duo = Group.objects.create(name="Duo")
+        band_hype = Group.objects.create(name="The Hype")
+        mary = Musician.objects.create(name="Mary Halvorson")
+        jonathan = Musician.objects.create(name="Jonathan Finlayson")
+        band_duo.members.set([mary, jonathan])
+        Concert.objects.create(name="Tiny desk concert", group=band_duo)
+        Concert.objects.create(name="Woodstock concert", group=band_hype)
+        # FK lookup.
+        concert_model_admin = ConcertAdmin(Concert, custom_site)
+        concert_model_admin.search_fields = ["group__name", "name"]
+        # Reverse FK lookup.
+        group_model_admin = GroupAdmin(Group, custom_site)
+        group_model_admin.search_fields = ["name", "concert__name", "members__name"]
+        for search_string, result_count in (
+            ("Duo Concert", 1),
+            ("Tiny Desk Concert", 1),
+            ("Concert", 2),
+            ("Other Concert", 0),
+            ("Duo Woodstock", 0),
+        ):
+            with self.subTest(search_string=search_string):
+                # FK lookup.
+                request = self.factory.get(
+                    "/concert/", data={SEARCH_VAR: search_string}
+                )
+                request.user = self.superuser
+                concert_changelist = concert_model_admin.get_changelist_instance(
+                    request
+                )
+                self.assertEqual(concert_changelist.queryset.count(), result_count)
+                # Reverse FK lookup.
+                request = self.factory.get("/group/", data={SEARCH_VAR: search_string})
+                request.user = self.superuser
+                group_changelist = group_model_admin.get_changelist_instance(request)
+                self.assertEqual(group_changelist.queryset.count(), result_count)
+        # Many-to-many lookup.
+        for search_string, result_count in (
+            ("Finlayson Duo Tiny", 1),
+            ("Finlayson", 1),
+            ("Finlayson Hype", 0),
+            ("Jonathan Finlayson Duo", 1),
+            ("Mary Jonathan Duo", 0),
+            ("Oscar Finlayson Duo", 0),
+        ):
+            with self.subTest(search_string=search_string):
+                request = self.factory.get("/group/", data={SEARCH_VAR: search_string})
+                request.user = self.superuser
+                group_changelist = group_model_admin.get_changelist_instance(request)
+                self.assertEqual(group_changelist.queryset.count(), result_count)
+
     def test_pk_in_search_fields(self):
         band = Group.objects.create(name="The Hype")
         Concert.objects.create(name="Woodstock", group=band)
@@ -1431,7 +1523,7 @@ class ChangeListTests(TestCase):
         request = self._mocked_authenticated_request("/band/", superuser)
         response = m.changelist_view(request)
         self.assertIsNone(response.context_data["cl"].search_help_text)
-        self.assertNotContains(response, '<div class="help">')
+        self.assertNotContains(response, '<div class="help id="searchbar_helptext">')
         # search_fields with search_help_text.
         m.search_help_text = "Search help text"
         request = self._mocked_authenticated_request("/band/", superuser)
@@ -1439,7 +1531,14 @@ class ChangeListTests(TestCase):
         self.assertEqual(
             response.context_data["cl"].search_help_text, "Search help text"
         )
-        self.assertContains(response, '<div class="help">Search help text</div>')
+        self.assertContains(
+            response, '<div class="help" id="searchbar_helptext">Search help text</div>'
+        )
+        self.assertContains(
+            response,
+            '<input type="text" size="40" name="q" value="" id="searchbar" '
+            'autofocus aria-describedby="searchbar_helptext">',
+        )
 
 
 class GetAdminLogTests(TestCase):
@@ -1505,24 +1604,27 @@ class SeleniumTests(AdminSeleniumTestCase):
         """
         The status line for selected rows gets updated correctly (#22038).
         """
+        from selenium.webdriver.common.by import By
+
         self.admin_login(username="super", password="secret")
         self.selenium.get(self.live_server_url + reverse("admin:auth_user_changelist"))
 
         form_id = "#changelist-form"
 
         # Test amount of rows in the Changelist
-        rows = self.selenium.find_elements_by_css_selector(
-            "%s #result_list tbody tr" % form_id
+        rows = self.selenium.find_elements(
+            By.CSS_SELECTOR, "%s #result_list tbody tr" % form_id
         )
         self.assertEqual(len(rows), 1)
         row = rows[0]
 
-        selection_indicator = self.selenium.find_element_by_css_selector(
-            "%s .action-counter" % form_id
+        selection_indicator = self.selenium.find_element(
+            By.CSS_SELECTOR, "%s .action-counter" % form_id
         )
-        all_selector = self.selenium.find_element_by_id("action-toggle")
-        row_selector = self.selenium.find_element_by_css_selector(
-            "%s #result_list tbody tr:first-child .action-select" % form_id
+        all_selector = self.selenium.find_element(By.ID, "action-toggle")
+        row_selector = self.selenium.find_element(
+            By.CSS_SELECTOR,
+            "%s #result_list tbody tr:first-child .action-select" % form_id,
         )
 
         # Test current selection
@@ -1548,6 +1650,7 @@ class SeleniumTests(AdminSeleniumTestCase):
         should select all rows in-between.
         """
         from selenium.webdriver.common.action_chains import ActionChains
+        from selenium.webdriver.common.by import By
         from selenium.webdriver.common.keys import Keys
 
         Parent.objects.bulk_create([Parent(name="parent%d" % i) for i in range(5)])
@@ -1555,8 +1658,8 @@ class SeleniumTests(AdminSeleniumTestCase):
         self.selenium.get(
             self.live_server_url + reverse("admin:admin_changelist_parent_changelist")
         )
-        checkboxes = self.selenium.find_elements_by_css_selector(
-            "tr input.action-select"
+        checkboxes = self.selenium.find_elements(
+            By.CSS_SELECTOR, "tr input.action-select"
         )
         self.assertEqual(len(checkboxes), 5)
         for c in checkboxes:
@@ -1571,22 +1674,24 @@ class SeleniumTests(AdminSeleniumTestCase):
         self.assertIs(checkboxes[-1].get_property("checked"), False)
 
     def test_select_all_across_pages(self):
+        from selenium.webdriver.common.by import By
+
         Parent.objects.bulk_create([Parent(name="parent%d" % i) for i in range(101)])
         self.admin_login(username="super", password="secret")
         self.selenium.get(
             self.live_server_url + reverse("admin:admin_changelist_parent_changelist")
         )
 
-        selection_indicator = self.selenium.find_element_by_css_selector(
-            ".action-counter"
+        selection_indicator = self.selenium.find_element(
+            By.CSS_SELECTOR, ".action-counter"
         )
-        select_all_indicator = self.selenium.find_element_by_css_selector(
-            ".actions .all"
+        select_all_indicator = self.selenium.find_element(
+            By.CSS_SELECTOR, ".actions .all"
         )
-        question = self.selenium.find_element_by_css_selector(".actions > .question")
-        clear = self.selenium.find_element_by_css_selector(".actions > .clear")
-        select_all = self.selenium.find_element_by_id("action-toggle")
-        select_across = self.selenium.find_elements_by_name("select_across")
+        question = self.selenium.find_element(By.CSS_SELECTOR, ".actions > .question")
+        clear = self.selenium.find_element(By.CSS_SELECTOR, ".actions > .clear")
+        select_all = self.selenium.find_element(By.ID, "action-toggle")
+        select_across = self.selenium.find_elements(By.NAME, "select_across")
 
         self.assertIs(question.is_displayed(), False)
         self.assertIs(clear.is_displayed(), False)
@@ -1627,6 +1732,8 @@ class SeleniumTests(AdminSeleniumTestCase):
         self.assertIs(select_all_indicator.is_displayed(), False)
 
     def test_actions_warn_on_pending_edits(self):
+        from selenium.webdriver.common.by import By
+
         Parent.objects.create(name="foo")
 
         self.admin_login(username="super", password="secret")
@@ -1634,11 +1741,11 @@ class SeleniumTests(AdminSeleniumTestCase):
             self.live_server_url + reverse("admin:admin_changelist_parent_changelist")
         )
 
-        name_input = self.selenium.find_element_by_id("id_form-0-name")
+        name_input = self.selenium.find_element(By.ID, "id_form-0-name")
         name_input.clear()
         name_input.send_keys("bar")
-        self.selenium.find_element_by_id("action-toggle").click()
-        self.selenium.find_element_by_name("index").click()  # Go
+        self.selenium.find_element(By.ID, "action-toggle").click()
+        self.selenium.find_element(By.NAME, "index").click()  # Go
         alert = self.selenium.switch_to.alert
         try:
             self.assertEqual(
@@ -1650,6 +1757,7 @@ class SeleniumTests(AdminSeleniumTestCase):
             alert.dismiss()
 
     def test_save_with_changes_warns_on_pending_action(self):
+        from selenium.webdriver.common.by import By
         from selenium.webdriver.support.ui import Select
 
         Parent.objects.create(name="parent")
@@ -1659,13 +1767,13 @@ class SeleniumTests(AdminSeleniumTestCase):
             self.live_server_url + reverse("admin:admin_changelist_parent_changelist")
         )
 
-        name_input = self.selenium.find_element_by_id("id_form-0-name")
+        name_input = self.selenium.find_element(By.ID, "id_form-0-name")
         name_input.clear()
         name_input.send_keys("other name")
-        Select(self.selenium.find_element_by_name("action")).select_by_value(
+        Select(self.selenium.find_element(By.NAME, "action")).select_by_value(
             "delete_selected"
         )
-        self.selenium.find_element_by_name("_save").click()
+        self.selenium.find_element(By.NAME, "_save").click()
         alert = self.selenium.switch_to.alert
         try:
             self.assertEqual(
@@ -1678,6 +1786,7 @@ class SeleniumTests(AdminSeleniumTestCase):
             alert.dismiss()
 
     def test_save_without_changes_warns_on_pending_action(self):
+        from selenium.webdriver.common.by import By
         from selenium.webdriver.support.ui import Select
 
         Parent.objects.create(name="parent")
@@ -1687,10 +1796,10 @@ class SeleniumTests(AdminSeleniumTestCase):
             self.live_server_url + reverse("admin:admin_changelist_parent_changelist")
         )
 
-        Select(self.selenium.find_element_by_name("action")).select_by_value(
+        Select(self.selenium.find_element(By.NAME, "action")).select_by_value(
             "delete_selected"
         )
-        self.selenium.find_element_by_name("_save").click()
+        self.selenium.find_element(By.NAME, "_save").click()
         alert = self.selenium.switch_to.alert
         try:
             self.assertEqual(
@@ -1701,3 +1810,55 @@ class SeleniumTests(AdminSeleniumTestCase):
             )
         finally:
             alert.dismiss()
+
+    def test_collapse_filters(self):
+        from selenium.webdriver.common.by import By
+
+        self.admin_login(username="super", password="secret")
+        self.selenium.get(self.live_server_url + reverse("admin:auth_user_changelist"))
+
+        # The UserAdmin has 3 field filters by default: "staff status",
+        # "superuser status", and "active".
+        details = self.selenium.find_elements(By.CSS_SELECTOR, "details")
+        # All filters are opened at first.
+        for detail in details:
+            self.assertTrue(detail.get_attribute("open"))
+        # Collapse "staff' and "superuser" filters.
+        for detail in details[:2]:
+            summary = detail.find_element(By.CSS_SELECTOR, "summary")
+            summary.click()
+            self.assertFalse(detail.get_attribute("open"))
+        # Filters are in the same state after refresh.
+        self.selenium.refresh()
+        self.assertFalse(
+            self.selenium.find_element(
+                By.CSS_SELECTOR, "[data-filter-title='staff status']"
+            ).get_attribute("open")
+        )
+        self.assertFalse(
+            self.selenium.find_element(
+                By.CSS_SELECTOR, "[data-filter-title='superuser status']"
+            ).get_attribute("open")
+        )
+        self.assertTrue(
+            self.selenium.find_element(
+                By.CSS_SELECTOR, "[data-filter-title='active']"
+            ).get_attribute("open")
+        )
+        # Collapse a filter on another view (Bands).
+        self.selenium.get(
+            self.live_server_url + reverse("admin:admin_changelist_band_changelist")
+        )
+        self.selenium.find_element(By.CSS_SELECTOR, "summary").click()
+        # Go to Users view and then, back again to Bands view.
+        self.selenium.get(self.live_server_url + reverse("admin:auth_user_changelist"))
+        self.selenium.get(
+            self.live_server_url + reverse("admin:admin_changelist_band_changelist")
+        )
+        # The filter remains in the same state.
+        self.assertFalse(
+            self.selenium.find_element(
+                By.CSS_SELECTOR,
+                "[data-filter-title='number of members']",
+            ).get_attribute("open")
+        )
