@@ -32,12 +32,12 @@ class ModelInstanceCreationTests(TestCase):
             pub_date=datetime(2005, 7, 28),
         )
         self.assertIsNone(a.id)
-        self.assertEqual(Article.objects.all().count(), 0)
+        self.assertEqual(Article.objects.count(), 0)
 
         # Save it into the database. You have to call save() explicitly.
         a.save()
         self.assertIsNotNone(a.id)
-        self.assertEqual(Article.objects.all().count(), 1)
+        self.assertEqual(Article.objects.count(), 1)
 
     def test_can_initialize_model_instance_using_positional_arguments(self):
         """
@@ -90,15 +90,23 @@ class ModelInstanceCreationTests(TestCase):
             Article(None, "Seventh article", datetime(2021, 3, 1), pub_date=None)
 
     def test_cannot_create_instance_with_invalid_kwargs(self):
-        with self.assertRaisesMessage(
-            TypeError, "Article() got an unexpected keyword argument 'foo'"
-        ):
+        msg = "Article() got unexpected keyword arguments: 'foo'"
+        with self.assertRaisesMessage(TypeError, msg):
             Article(
                 id=None,
                 headline="Some headline",
                 pub_date=datetime(2005, 7, 31),
                 foo="bar",
             )
+        msg = "Article() got unexpected keyword arguments: 'foo', 'bar'"
+        with self.assertRaisesMessage(TypeError, msg):
+            Article(
+                id=None,
+                headline="Some headline",
+                pub_date=datetime(2005, 7, 31),
+                foo="bar",
+                bar="baz",
+            )
 
     def test_can_leave_off_value_for_autofield_and_it_gets_value_on_save(self):
         """
@@ -191,7 +199,7 @@ class ModelTest(TestCase):
         for headline in headlines:
             Article(headline=headline, pub_date=some_pub_date).save()
         self.assertQuerysetEqual(
-            Article.objects.all().order_by("headline"),
+            Article.objects.order_by("headline"),
             sorted(headlines),
             transform=lambda a: a.headline,
         )
@@ -694,6 +702,24 @@ class ManagerTest(SimpleTestCase):
         "union",
         "intersection",
         "difference",
+        "aaggregate",
+        "abulk_create",
+        "abulk_update",
+        "acontains",
+        "acount",
+        "acreate",
+        "aearliest",
+        "aexists",
+        "aexplain",
+        "afirst",
+        "aget",
+        "aget_or_create",
+        "ain_bulk",
+        "aiterator",
+        "alast",
+        "alatest",
+        "aupdate",
+        "aupdate_or_create",
     ]
 
     def test_manager_methods(self):
