@@ -69,6 +69,7 @@ from django.test.utils import (
     isolate_apps,
     register_lookup,
 )
+from django.utils.deprecation import RemovedInDjango50Warning
 from django.utils.functional import SimpleLazyObject
 
 from .models import (
@@ -607,6 +608,15 @@ class BasicExpressionsTests(TestCase):
             qs.query.annotations["small_company"],
         )
 
+    def test_subquery_sql(self):
+        employees = Employee.objects.all()
+        employees_subquery = Subquery(employees)
+        self.assertIs(employees_subquery.query.subquery, True)
+        self.assertIs(employees.query.subquery, False)
+        compiler = employees_subquery.query.get_compiler(connection=connection)
+        sql, _ = employees_subquery.as_sql(compiler, connection)
+        self.assertIn("(SELECT ", sql)
+
     def test_in_subquery(self):
         # This is a contrived test (and you really wouldn't write this query),
         # but it is a succinct way to test the __in=Subquery() construct.
@@ -1102,11 +1112,6 @@ class IterableLookupInnerExpressionsTests(TestCase):
             experiment=experiment_2,
             result_time=datetime.datetime(2016, 1, 8, 5, 0, 0),
         )
-
-        within_experiment_time = [F("experiment__start"), F("experiment__end")]
-        queryset = Result.objects.filter(result_time__range=within_experiment_time)
-        self.assertSequenceEqual(queryset, [r1])
-
         within_experiment_time = [F("experiment__start"), F("experiment__end")]
         queryset = Result.objects.filter(result_time__range=within_experiment_time)
         self.assertSequenceEqual(queryset, [r1])
@@ -1121,7 +1126,7 @@ class FTests(SimpleTestCase):
     def test_deconstruct(self):
         f = F("name")
         path, args, kwargs = f.deconstruct()
-        self.assertEqual(path, "django.db.models.expressions.F")
+        self.assertEqual(path, "django.db.models.F")
         self.assertEqual(args, (f.name,))
         self.assertEqual(kwargs, {})
 
@@ -1316,6 +1321,13 @@ class ExpressionsNumericTests(TestCase):
             ordered=False,
         )
 
+    def test_filter_decimal_expression(self):
+        obj = Number.objects.create(integer=0, float=1, decimal_value=Decimal("1"))
+        qs = Number.objects.annotate(
+            x=ExpressionWrapper(Value(1), output_field=DecimalField()),
+        ).filter(Q(x=1, integer=0) & Q(x=Decimal("1")))
+        self.assertSequenceEqual(qs, [obj])
+
     def test_complex_expressions(self):
         """
         Complex expressions of different connection types are possible.
@@ -1394,6 +1406,12 @@ class ExpressionOperatorTests(TestCase):
         Number.objects.filter(pk=self.n.pk).update(integer=F("integer") % 20)
         self.assertEqual(Number.objects.get(pk=self.n.pk).integer, 2)
 
+    def test_lefthand_modulo_null(self):
+        # LH Modulo arithmetic on integers.
+        Employee.objects.create(firstname="John", lastname="Doe", salary=None)
+        qs = Employee.objects.annotate(modsalary=F("salary") % 20)
+        self.assertIsNone(qs.get().salary)
+
     def test_lefthand_bitwise_and(self):
         # LH Bitwise ands on integers
         Number.objects.filter(pk=self.n.pk).update(integer=F("integer").bitand(56))
@@ -1446,6 +1464,12 @@ class ExpressionOperatorTests(TestCase):
         employee.refresh_from_db()
         self.assertIsNone(employee.salary)
 
+    def test_lefthand_bitwise_xor_right_null(self):
+        employee = Employee.objects.create(firstname="John", lastname="Doe", salary=48)
+        Employee.objects.update(salary=F("salary").bitxor(None))
+        employee.refresh_from_db()
+        self.assertIsNone(employee.salary)
+
     @unittest.skipUnless(
         connection.vendor == "oracle", "Oracle doesn't support bitwise XOR."
     )
@@ -1701,7 +1725,39 @@ class FTimeDeltaTests(TestCase):
             ]
             self.assertEqual(test_set, self.expnames[: i + 1])
 
-    @skipUnlessDBFeature("supports_mixed_date_datetime_comparisons")
+    def test_datetime_and_durationfield_addition_with_filter(self):
+        test_set = Experiment.objects.filter(end=F("start") + F("estimated_time"))
+        self.assertGreater(test_set.count(), 0)
+        self.assertEqual(
+            [e.name for e in test_set],
+            [
+                e.name
+                for e in Experiment.objects.all()
+                if e.end == e.start + e.estimated_time
+            ],
+        )
+
+    def test_datetime_and_duration_field_addition_with_annotate_and_no_output_field(
+        self,
+    ):
+        test_set = Experiment.objects.annotate(
+            estimated_end=F("start") + F("estimated_time")
+        )
+        self.assertEqual(
+            [e.estimated_end for e in test_set],
+            [e.start + e.estimated_time for e in test_set],
+        )
+
+    @skipUnlessDBFeature("supports_temporal_subtraction")
+    def test_datetime_subtraction_with_annotate_and_no_output_field(self):
+        test_set = Experiment.objects.annotate(
+            calculated_duration=F("end") - F("start")
+        )
+        self.assertEqual(
+            [e.calculated_duration for e in test_set],
+            [e.end - e.start for e in test_set],
+        )
+
     def test_mixed_comparisons1(self):
         for i, delay in enumerate(self.delays):
             test_set = [
@@ -2037,14 +2093,14 @@ class ValueTests(TestCase):
     def test_deconstruct(self):
         value = Value("name")
         path, args, kwargs = value.deconstruct()
-        self.assertEqual(path, "django.db.models.expressions.Value")
+        self.assertEqual(path, "django.db.models.Value")
         self.assertEqual(args, (value.value,))
         self.assertEqual(kwargs, {})
 
     def test_deconstruct_output_field(self):
         value = Value("name", output_field=CharField())
         path, args, kwargs = value.deconstruct()
-        self.assertEqual(path, "django.db.models.expressions.Value")
+        self.assertEqual(path, "django.db.models.Value")
         self.assertEqual(args, (value.value,))
         self.assertEqual(len(kwargs), 1)
         self.assertEqual(
@@ -2128,6 +2184,30 @@ class ValueTests(TestCase):
         with self.assertRaisesMessage(FieldError, msg):
             Value(object()).output_field
 
+    def test_output_field_does_not_create_broken_validators(self):
+        """
+        The output field for a given Value doesn't get cleaned & validated,
+        however validators may still be instantiated for a given field type
+        and this demonstrates that they don't throw an exception.
+        """
+        value_types = [
+            "str",
+            True,
+            42,
+            3.14,
+            datetime.date(2019, 5, 15),
+            datetime.datetime(2019, 5, 15),
+            datetime.time(3, 16),
+            datetime.timedelta(1),
+            Decimal("3.14"),
+            b"",
+            uuid.uuid4(),
+        ]
+        for value in value_types:
+            with self.subTest(type=type(value)):
+                field = Value(value)._resolve_output_field()
+                field.clean(value, model_instance=None)
+
 
 class ExistsTests(TestCase):
     def test_optimizations(self):
@@ -2152,6 +2232,19 @@ class ExistsTests(TestCase):
         )
         self.assertNotIn("ORDER BY", captured_sql)
 
+    def test_negated_empty_exists(self):
+        manager = Manager.objects.create()
+        qs = Manager.objects.filter(~Exists(Manager.objects.none()) & Q(pk=manager.pk))
+        self.assertSequenceEqual(qs, [manager])
+
+    def test_select_negated_empty_exists(self):
+        manager = Manager.objects.create()
+        qs = Manager.objects.annotate(
+            not_exists=~Exists(Manager.objects.none())
+        ).filter(pk=manager.pk)
+        self.assertSequenceEqual(qs, [manager])
+        self.assertIs(qs.get().not_exists, True)
+
 
 class FieldTransformTests(TestCase):
     @classmethod
@@ -2280,7 +2373,9 @@ class ReprTests(SimpleTestCase):
 
 
 class CombinableTests(SimpleTestCase):
-    bitwise_msg = "Use .bitand() and .bitor() for bitwise logical operations."
+    bitwise_msg = (
+        "Use .bitand(), .bitor(), and .bitxor() for bitwise logical operations."
+    )
 
     def test_negation(self):
         c = Combinable()
@@ -2294,6 +2389,10 @@ class CombinableTests(SimpleTestCase):
         with self.assertRaisesMessage(NotImplementedError, self.bitwise_msg):
             Combinable() | Combinable()
 
+    def test_xor(self):
+        with self.assertRaisesMessage(NotImplementedError, self.bitwise_msg):
+            Combinable() ^ Combinable()
+
     def test_reversed_and(self):
         with self.assertRaisesMessage(NotImplementedError, self.bitwise_msg):
             object() & Combinable()
@@ -2302,9 +2401,13 @@ class CombinableTests(SimpleTestCase):
         with self.assertRaisesMessage(NotImplementedError, self.bitwise_msg):
             object() | Combinable()
 
+    def test_reversed_xor(self):
+        with self.assertRaisesMessage(NotImplementedError, self.bitwise_msg):
+            object() ^ Combinable()
+
 
 class CombinedExpressionTests(SimpleTestCase):
-    def test_resolve_output_field(self):
+    def test_resolve_output_field_number(self):
         tests = [
             (IntegerField, AutoField, IntegerField),
             (AutoField, IntegerField, IntegerField),
@@ -2326,6 +2429,92 @@ class CombinedExpressionTests(SimpleTestCase):
                     )
                     self.assertIsInstance(expr.output_field, combined)
 
+    def test_resolve_output_field_with_null(self):
+        def null():
+            return Value(None)
+
+        tests = [
+            # Numbers.
+            (AutoField, Combinable.ADD, null),
+            (DecimalField, Combinable.ADD, null),
+            (FloatField, Combinable.ADD, null),
+            (IntegerField, Combinable.ADD, null),
+            (IntegerField, Combinable.SUB, null),
+            (null, Combinable.ADD, IntegerField),
+            # Dates.
+            (DateField, Combinable.ADD, null),
+            (DateTimeField, Combinable.ADD, null),
+            (DurationField, Combinable.ADD, null),
+            (TimeField, Combinable.ADD, null),
+            (TimeField, Combinable.SUB, null),
+            (null, Combinable.ADD, DateTimeField),
+            (DateField, Combinable.SUB, null),
+        ]
+        for lhs, connector, rhs in tests:
+            msg = (
+                f"Cannot infer type of {connector!r} expression involving these types: "
+            )
+            with self.subTest(lhs=lhs, connector=connector, rhs=rhs):
+                expr = CombinedExpression(
+                    Expression(lhs()),
+                    connector,
+                    Expression(rhs()),
+                )
+                with self.assertRaisesMessage(FieldError, msg):
+                    expr.output_field
+
+    def test_resolve_output_field_dates(self):
+        tests = [
+            # Add - same type.
+            (DateField, Combinable.ADD, DateField, FieldError),
+            (DateTimeField, Combinable.ADD, DateTimeField, FieldError),
+            (TimeField, Combinable.ADD, TimeField, FieldError),
+            (DurationField, Combinable.ADD, DurationField, DurationField),
+            # Add - different type.
+            (DateField, Combinable.ADD, DurationField, DateTimeField),
+            (DateTimeField, Combinable.ADD, DurationField, DateTimeField),
+            (TimeField, Combinable.ADD, DurationField, TimeField),
+            (DurationField, Combinable.ADD, DateField, DateTimeField),
+            (DurationField, Combinable.ADD, DateTimeField, DateTimeField),
+            (DurationField, Combinable.ADD, TimeField, TimeField),
+            # Subtract - same type.
+            (DateField, Combinable.SUB, DateField, DurationField),
+            (DateTimeField, Combinable.SUB, DateTimeField, DurationField),
+            (TimeField, Combinable.SUB, TimeField, DurationField),
+            (DurationField, Combinable.SUB, DurationField, DurationField),
+            # Subtract - different type.
+            (DateField, Combinable.SUB, DurationField, DateTimeField),
+            (DateTimeField, Combinable.SUB, DurationField, DateTimeField),
+            (TimeField, Combinable.SUB, DurationField, TimeField),
+            (DurationField, Combinable.SUB, DateField, FieldError),
+            (DurationField, Combinable.SUB, DateTimeField, FieldError),
+            (DurationField, Combinable.SUB, DateTimeField, FieldError),
+        ]
+        for lhs, connector, rhs, combined in tests:
+            msg = (
+                f"Cannot infer type of {connector!r} expression involving these types: "
+            )
+            with self.subTest(lhs=lhs, connector=connector, rhs=rhs, combined=combined):
+                expr = CombinedExpression(
+                    Expression(lhs()),
+                    connector,
+                    Expression(rhs()),
+                )
+                if issubclass(combined, Exception):
+                    with self.assertRaisesMessage(combined, msg):
+                        expr.output_field
+                else:
+                    self.assertIsInstance(expr.output_field, combined)
+
+    def test_mixed_char_date_with_annotate(self):
+        queryset = Experiment.objects.annotate(nonsense=F("name") + F("assigned"))
+        msg = (
+            "Cannot infer type of '+' expression involving these types: CharField, "
+            "DateField. You must set output_field."
+        )
+        with self.assertRaisesMessage(FieldError, msg):
+            list(queryset)
+
 
 class ExpressionWrapperTests(SimpleTestCase):
     def test_empty_group_by(self):
@@ -2349,7 +2538,7 @@ class OrderByTests(SimpleTestCase):
         )
         self.assertNotEqual(
             OrderBy(F("field"), nulls_last=True),
-            OrderBy(F("field"), nulls_last=False),
+            OrderBy(F("field")),
         )
 
     def test_hash(self):
@@ -2359,5 +2548,22 @@ class OrderByTests(SimpleTestCase):
         )
         self.assertNotEqual(
             hash(OrderBy(F("field"), nulls_last=True)),
-            hash(OrderBy(F("field"), nulls_last=False)),
+            hash(OrderBy(F("field"))),
         )
+
+    def test_nulls_false(self):
+        # These tests will catch ValueError in Django 5.0 when passing False to
+        # nulls_first and nulls_last becomes forbidden.
+        # msg = "nulls_first and nulls_last values must be True or None."
+        msg = (
+            "Passing nulls_first=False or nulls_last=False is deprecated, use None "
+            "instead."
+        )
+        with self.assertRaisesMessage(RemovedInDjango50Warning, msg):
+            OrderBy(F("field"), nulls_first=False)
+        with self.assertRaisesMessage(RemovedInDjango50Warning, msg):
+            OrderBy(F("field"), nulls_last=False)
+        with self.assertRaisesMessage(RemovedInDjango50Warning, msg):
+            F("field").asc(nulls_first=False)
+        with self.assertRaisesMessage(RemovedInDjango50Warning, msg):
+            F("field").desc(nulls_last=False)
