@@ -44,6 +44,7 @@ from .models import (
     M2MSimpleCircularA,
     M2MSimpleCircularB,
     M2MThroughAB,
+    NaturalKeyWithFKDependency,
     NKChild,
     Parent,
     Person,
@@ -791,6 +792,25 @@ class NaturalKeyFixtureTests(TestCase):
         )
 
 
+class NaturalKeyFixtureOnOtherDatabaseTests(TestCase):
+    databases = {"other"}
+
+    def test_natural_key_dependencies(self):
+        """
+        Natural keys with foreing keys in dependencies works in a multiple
+        database setup.
+        """
+        management.call_command(
+            "loaddata",
+            "nk_with_foreign_key.json",
+            database="other",
+            verbosity=0,
+        )
+        obj = NaturalKeyWithFKDependency.objects.using("other").get()
+        self.assertEqual(obj.name, "The Lord of the Rings")
+        self.assertEqual(obj.author.name, "J.R.R. Tolkien")
+
+
 class M2MNaturalKeyFixtureTests(TestCase):
     """Tests for ticket #14426."""
 
