@@ -1,6 +1,8 @@
 from django.forms import CharField, Form, Media, MultiWidget, TextInput
 from django.template import Context, Template
+from django.templatetags.static import static
 from django.test import SimpleTestCase, override_settings
+from django.utils.html import format_html, html_safe
 
 
 @override_settings(
@@ -21,9 +23,9 @@ class FormsMediaTestCase(SimpleTestCase):
         )
         self.assertEqual(
             str(m),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="https://secure.other.com/path/to/js3"></script>',
@@ -46,9 +48,9 @@ class FormsMediaTestCase(SimpleTestCase):
         m3 = Media(Foo)
         self.assertEqual(
             str(m3),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="https://secure.other.com/path/to/js3"></script>',
@@ -81,9 +83,9 @@ class FormsMediaTestCase(SimpleTestCase):
         w1 = MyWidget1()
         self.assertEqual(
             str(w1.media),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="https://secure.other.com/path/to/js3"></script>',
@@ -92,9 +94,9 @@ class FormsMediaTestCase(SimpleTestCase):
         # Media objects can be interrogated by media type
         self.assertEqual(
             str(w1.media["css"]),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">',
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">',
         )
 
         self.assertEqual(
@@ -131,10 +133,10 @@ class FormsMediaTestCase(SimpleTestCase):
         w3 = MyWidget3()
         self.assertEqual(
             str(w1.media + w2.media + w3.media),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css3" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="/path/to/js4"></script>\n'
@@ -144,9 +146,9 @@ class FormsMediaTestCase(SimpleTestCase):
         # media addition hasn't affected the original objects
         self.assertEqual(
             str(w1.media),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="https://secure.other.com/path/to/js3"></script>',
@@ -163,7 +165,7 @@ class FormsMediaTestCase(SimpleTestCase):
         w4 = MyWidget4()
         self.assertEqual(
             str(w4.media),
-            """<link href="/path/to/css1" type="text/css" media="all" rel="stylesheet">
+            """<link href="/path/to/css1" media="all" rel="stylesheet">
 <script src="/path/to/js1"></script>""",
         )
 
@@ -177,7 +179,7 @@ class FormsMediaTestCase(SimpleTestCase):
         )
         self.assertEqual(
             str(media),
-            """<link href="/path/to/css1" type="text/css" media="all" rel="stylesheet">
+            """<link href="/path/to/css1" media="all" rel="stylesheet">
 <script src="/path/to/js1"></script>""",
         )
 
@@ -196,7 +198,7 @@ class FormsMediaTestCase(SimpleTestCase):
         w4 = MyWidget4()
         self.assertEqual(
             str(w4.media),
-            """<link href="/some/path" type="text/css" media="all" rel="stylesheet">
+            """<link href="/some/path" media="all" rel="stylesheet">
 <script src="/some/js"></script>""",
         )
 
@@ -212,8 +214,8 @@ class FormsMediaTestCase(SimpleTestCase):
         w5 = MyWidget5()
         self.assertEqual(
             str(w5.media),
-            """<link href="/some/path" type="text/css" media="all" rel="stylesheet">
-<link href="/other/path" type="text/css" media="all" rel="stylesheet">
+            """<link href="/some/path" media="all" rel="stylesheet">
+<link href="/other/path" media="all" rel="stylesheet">
 <script src="/some/js"></script>
 <script src="/other/js"></script>""",
         )
@@ -241,10 +243,10 @@ class FormsMediaTestCase(SimpleTestCase):
         w6 = MyWidget6()
         self.assertEqual(
             str(w6.media),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/other/path" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/other/path" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="/other/js"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
@@ -273,9 +275,9 @@ class FormsMediaTestCase(SimpleTestCase):
         w7 = MyWidget7()
         self.assertEqual(
             str(w7.media),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="https://secure.other.com/path/to/js3"></script>',
@@ -291,14 +293,13 @@ class FormsMediaTestCase(SimpleTestCase):
         w8 = MyWidget8()
         self.assertEqual(
             str(w8.media),
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
-            '<script src="/path/to/js1"></script>\n'
-            '<script src="http://media.other.com/path/to/js2"></script>\n'
-            '<script src="/path/to/js4"></script>\n'
-            '<script src="https://secure.other.com/path/to/js3"></script>',
+            """<link href="/path/to/css3" media="all" rel="stylesheet">
+<link href="http://media.example.com/static/path/to/css1" media="all" rel="stylesheet">
+<link href="/path/to/css2" media="all" rel="stylesheet">
+<script src="/path/to/js1"></script>
+<script src="http://media.other.com/path/to/js2"></script>
+<script src="/path/to/js4"></script>
+<script src="https://secure.other.com/path/to/js3"></script>""",
         )
 
     def test_media_inheritance_from_property(self):
@@ -327,8 +328,8 @@ class FormsMediaTestCase(SimpleTestCase):
         w9 = MyWidget9()
         self.assertEqual(
             str(w9.media),
-            """<link href="/some/path" type="text/css" media="all" rel="stylesheet">
-<link href="/other/path" type="text/css" media="all" rel="stylesheet">
+            """<link href="/some/path" media="all" rel="stylesheet">
+<link href="/other/path" media="all" rel="stylesheet">
 <script src="/some/js"></script>
 <script src="/other/js"></script>""",
         )
@@ -343,11 +344,10 @@ class FormsMediaTestCase(SimpleTestCase):
         w10 = MyWidget10()
         self.assertEqual(
             str(w10.media),
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<script src="/path/to/js1"></script>\n'
-            '<script src="/path/to/js4"></script>',
+            """<link href="/path/to/css3" media="all" rel="stylesheet">
+<link href="http://media.example.com/static/path/to/css1" media="all" rel="stylesheet">
+<script src="/path/to/js1"></script>
+<script src="/path/to/js4"></script>""",
         )
 
     def test_media_inheritance_extends(self):
@@ -371,14 +371,13 @@ class FormsMediaTestCase(SimpleTestCase):
         w11 = MyWidget11()
         self.assertEqual(
             str(w11.media),
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
-            '<script src="/path/to/js1"></script>\n'
-            '<script src="http://media.other.com/path/to/js2"></script>\n'
-            '<script src="/path/to/js4"></script>\n'
-            '<script src="https://secure.other.com/path/to/js3"></script>',
+            """<link href="/path/to/css3" media="all" rel="stylesheet">
+<link href="http://media.example.com/static/path/to/css1" media="all" rel="stylesheet">
+<link href="/path/to/css2" media="all" rel="stylesheet">
+<script src="/path/to/js1"></script>
+<script src="http://media.other.com/path/to/js2"></script>
+<script src="/path/to/js4"></script>
+<script src="https://secure.other.com/path/to/js3"></script>""",
         )
 
     def test_media_inheritance_single_type(self):
@@ -402,12 +401,11 @@ class FormsMediaTestCase(SimpleTestCase):
         w12 = MyWidget12()
         self.assertEqual(
             str(w12.media),
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
-            '<script src="/path/to/js1"></script>\n'
-            '<script src="/path/to/js4"></script>',
+            """<link href="/path/to/css3" media="all" rel="stylesheet">
+<link href="http://media.example.com/static/path/to/css1" media="all" rel="stylesheet">
+<link href="/path/to/css2" media="all" rel="stylesheet">
+<script src="/path/to/js1"></script>
+<script src="/path/to/js4"></script>""",
         )
 
     def test_multi_media(self):
@@ -428,10 +426,10 @@ class FormsMediaTestCase(SimpleTestCase):
         multimedia = MultimediaWidget()
         self.assertEqual(
             str(multimedia.media),
-            """<link href="/file4" type="text/css" media="print" rel="stylesheet">
-<link href="/file3" type="text/css" media="screen" rel="stylesheet">
-<link href="/file1" type="text/css" media="screen, print" rel="stylesheet">
-<link href="/file2" type="text/css" media="screen, print" rel="stylesheet">
+            """<link href="/file4" media="print" rel="stylesheet">
+<link href="/file3" media="screen" rel="stylesheet">
+<link href="/file1" media="screen, print" rel="stylesheet">
+<link href="/file2" media="screen, print" rel="stylesheet">
 <script src="/path/to/js1"></script>
 <script src="/path/to/js4"></script>""",
         )
@@ -470,10 +468,10 @@ class FormsMediaTestCase(SimpleTestCase):
         mymulti = MyMultiWidget()
         self.assertEqual(
             str(mymulti.media),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css3" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="/path/to/js4"></script>\n'
@@ -512,10 +510,10 @@ class FormsMediaTestCase(SimpleTestCase):
         f1 = MyForm()
         self.assertEqual(
             str(f1.media),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css3" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="/path/to/js4"></script>\n'
@@ -529,10 +527,10 @@ class FormsMediaTestCase(SimpleTestCase):
         f2 = AnotherForm()
         self.assertEqual(
             str(f1.media + f2.media),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css3" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="/path/to/js4"></script>\n'
@@ -551,12 +549,11 @@ class FormsMediaTestCase(SimpleTestCase):
         f3 = FormWithMedia()
         self.assertEqual(
             str(f3.media),
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/some/form/css" type="text/css" media="all" '
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
             'rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">\n'
+            '<link href="/some/form/css" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css3" media="all" rel="stylesheet">\n'
             '<script src="/path/to/js1"></script>\n'
             '<script src="/some/form/javascript"></script>\n'
             '<script src="http://media.other.com/path/to/js2"></script>\n'
@@ -574,12 +571,11 @@ class FormsMediaTestCase(SimpleTestCase):
             '<script src="http://media.other.com/path/to/js2"></script>\n'
             '<script src="/path/to/js4"></script>\n'
             '<script src="https://secure.other.com/path/to/js3"></script>'
-            '<link href="http://media.example.com/static/path/to/css1" type="text/css" '
-            'media="all" rel="stylesheet">\n'
-            '<link href="/some/form/css" type="text/css" media="all" '
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
             'rel="stylesheet">\n'
-            '<link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">\n'
-            '<link href="/path/to/css3" type="text/css" media="all" rel="stylesheet">',
+            '<link href="/some/form/css" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css3" media="all" rel="stylesheet">',
         )
 
     def test_html_safe(self):
@@ -716,3 +712,160 @@ class FormsMediaTestCase(SimpleTestCase):
         merged = media + empty_media
         self.assertEqual(merged._css_lists, [{"screen": ["a.css"]}])
         self.assertEqual(merged._js_lists, [["a"]])
+
+
+@html_safe
+class Asset:
+    def __init__(self, path):
+        self.path = path
+
+    def __eq__(self, other):
+        return (self.__class__ == other.__class__ and self.path == other.path) or (
+            other.__class__ == str and self.path == other
+        )
+
+    def __hash__(self):
+        return hash(self.path)
+
+    def __str__(self):
+        return self.absolute_path(self.path)
+
+    def absolute_path(self, path):
+        """
+        Given a relative or absolute path to a static asset, return an absolute
+        path. An absolute path will be returned unchanged while a relative path
+        will be passed to django.templatetags.static.static().
+        """
+        if path.startswith(("http://", "https://", "/")):
+            return path
+        return static(path)
+
+    def __repr__(self):
+        return f"{self.path!r}"
+
+
+class CSS(Asset):
+    def __init__(self, path, medium):
+        super().__init__(path)
+        self.medium = medium
+
+    def __str__(self):
+        path = super().__str__()
+        return format_html(
+            '<link href="{}" media="{}" rel="stylesheet">',
+            self.absolute_path(path),
+            self.medium,
+        )
+
+
+class JS(Asset):
+    def __init__(self, path, integrity=None):
+        super().__init__(path)
+        self.integrity = integrity or ""
+
+    def __str__(self, integrity=None):
+        path = super().__str__()
+        template = '<script src="{}"%s></script>' % (
+            ' integrity="{}"' if self.integrity else "{}"
+        )
+        return format_html(template, self.absolute_path(path), self.integrity)
+
+
+@override_settings(
+    STATIC_URL="http://media.example.com/static/",
+)
+class FormsMediaObjectTestCase(SimpleTestCase):
+    """Media handling when media are objects instead of raw strings."""
+
+    def test_construction(self):
+        m = Media(
+            css={"all": (CSS("path/to/css1", "all"), CSS("/path/to/css2", "all"))},
+            js=(
+                JS("/path/to/js1"),
+                JS("http://media.other.com/path/to/js2"),
+                JS(
+                    "https://secure.other.com/path/to/js3",
+                    integrity="9d947b87fdeb25030d56d01f7aa75800",
+                ),
+            ),
+        )
+        self.assertEqual(
+            str(m),
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
+            '<script src="/path/to/js1"></script>\n'
+            '<script src="http://media.other.com/path/to/js2"></script>\n'
+            '<script src="https://secure.other.com/path/to/js3" '
+            'integrity="9d947b87fdeb25030d56d01f7aa75800"></script>',
+        )
+        self.assertEqual(
+            repr(m),
+            "Media(css={'all': ['path/to/css1', '/path/to/css2']}, "
+            "js=['/path/to/js1', 'http://media.other.com/path/to/js2', "
+            "'https://secure.other.com/path/to/js3'])",
+        )
+
+    def test_simplest_class(self):
+        @html_safe
+        class SimpleJS:
+            """The simplest possible asset class."""
+
+            def __str__(self):
+                return '<script src="https://example.org/asset.js" rel="stylesheet">'
+
+        m = Media(js=(SimpleJS(),))
+        self.assertEqual(
+            str(m),
+            '<script src="https://example.org/asset.js" rel="stylesheet">',
+        )
+
+    def test_combine_media(self):
+        class MyWidget1(TextInput):
+            class Media:
+                css = {"all": (CSS("path/to/css1", "all"), "/path/to/css2")}
+                js = (
+                    "/path/to/js1",
+                    "http://media.other.com/path/to/js2",
+                    "https://secure.other.com/path/to/js3",
+                    JS("/path/to/js4", integrity="9d947b87fdeb25030d56d01f7aa75800"),
+                )
+
+        class MyWidget2(TextInput):
+            class Media:
+                css = {"all": (CSS("/path/to/css2", "all"), "/path/to/css3")}
+                js = (JS("/path/to/js1"), "/path/to/js4")
+
+        w1 = MyWidget1()
+        w2 = MyWidget2()
+        self.assertEqual(
+            str(w1.media + w2.media),
+            '<link href="http://media.example.com/static/path/to/css1" media="all" '
+            'rel="stylesheet">\n'
+            '<link href="/path/to/css2" media="all" rel="stylesheet">\n'
+            '<link href="/path/to/css3" media="all" rel="stylesheet">\n'
+            '<script src="/path/to/js1"></script>\n'
+            '<script src="http://media.other.com/path/to/js2"></script>\n'
+            '<script src="https://secure.other.com/path/to/js3"></script>\n'
+            '<script src="/path/to/js4" integrity="9d947b87fdeb25030d56d01f7aa75800">'
+            "</script>",
+        )
+
+    def test_media_deduplication(self):
+        # The deduplication doesn't only happen at the point of merging two or
+        # more media objects.
+        media = Media(
+            css={
+                "all": (
+                    CSS("/path/to/css1", "all"),
+                    CSS("/path/to/css1", "all"),
+                    "/path/to/css1",
+                )
+            },
+            js=(JS("/path/to/js1"), JS("/path/to/js1"), "/path/to/js1"),
+        )
+        self.assertEqual(
+            str(media),
+            '<link href="/path/to/css1" media="all" rel="stylesheet">\n'
+            '<script src="/path/to/js1"></script>',
+        )
