@@ -1,4 +1,4 @@
-from django.forms import FileInput
+from django.forms import FileField, FileInput, Form
 
 from .base import WidgetTest
 
@@ -35,3 +35,16 @@ class FileInputTest(WidgetTest):
         # user to keep the existing, initial value.
         self.assertIs(self.widget.use_required_attribute(None), True)
         self.assertIs(self.widget.use_required_attribute("resume.txt"), False)
+
+    def test_fieldset(self):
+        class TestForm(Form):
+            template_name = "forms_tests/use_fieldset.html"
+            field = FileField(widget=self.widget)
+
+        form = TestForm()
+        self.assertIs(self.widget.use_fieldset, False)
+        self.assertHTMLEqual(
+            '<div><label for="id_field">Field:</label><input id="id_field" '
+            'name="field" required type="file"></div>',
+            form.render(),
+        )
