@@ -54,6 +54,11 @@ class ModelAdminTests(TestCase):
         ma = ModelAdmin(Band, self.site)
         self.assertEqual(str(ma), "modeladmin.ModelAdmin")
 
+    def test_default_attributes(self):
+        ma = ModelAdmin(Band, self.site)
+        self.assertEqual(ma.actions, ())
+        self.assertEqual(ma.inlines, ())
+
     # form/fields/fieldsets interaction ##############################
 
     def test_default_fields(self):
@@ -503,7 +508,7 @@ class ModelAdminTests(TestCase):
 
         self.assertHTMLEqual(
             str(form["main_band"]),
-            '<div class="related-widget-wrapper">'
+            '<div class="related-widget-wrapper" data-model-ref="band">'
             '<select name="main_band" id="id_main_band" required>'
             '<option value="" selected>---------</option>'
             '<option value="%d">The Beatles</option>'
@@ -526,7 +531,7 @@ class ModelAdminTests(TestCase):
 
         self.assertHTMLEqual(
             str(form["main_band"]),
-            '<div class="related-widget-wrapper">'
+            '<div class="related-widget-wrapper" data-model-ref="band">'
             '<select name="main_band" id="id_main_band" required>'
             '<option value="" selected>---------</option>'
             '<option value="%d">The Doors</option>'
