@@ -10,7 +10,8 @@ from django.core.serializers.base import PickleSerializer, ProgressBar
 from django.db import connection, transaction
 from django.http import HttpResponse
 from django.test import SimpleTestCase, override_settings, skipUnlessDBFeature
-from django.test.utils import Approximate
+from django.test.utils import Approximate, ignore_warnings
+from django.utils.deprecation import RemovedInDjango50Warning
 
 from .models import (
     Actor,
@@ -281,7 +282,7 @@ class SerializersTestBase:
         with self.assertNumQueries(3):
             serializers.serialize(
                 self.serializer_name,
-                Article.objects.all().prefetch_related("categories", "meta_data"),
+                Article.objects.prefetch_related("categories", "meta_data"),
             )
         # One query for the Article table, and two m2m queries for each
         # article.
@@ -360,7 +361,7 @@ class SerializersTestBase:
         for obj in deserial_objs:
             self.assertFalse(obj.object.id)
             obj.save()
-        self.assertEqual(Category.objects.all().count(), 5)
+        self.assertEqual(Category.objects.count(), 5)
 
     def test_deterministic_mapping_ordering(self):
         """Mapping such as fields should be deterministically ordered. (#24558)"""
@@ -453,13 +454,14 @@ class SerializersTransactionTestBase:
                     obj.save()
 
         for model_cls in (Category, Author, Article):
-            self.assertEqual(model_cls.objects.all().count(), 1)
+            self.assertEqual(model_cls.objects.count(), 1)
         art_obj = Article.objects.all()[0]
-        self.assertEqual(art_obj.categories.all().count(), 1)
+        self.assertEqual(art_obj.categories.count(), 1)
         self.assertEqual(art_obj.author.name, "Agnes")
 
 
 class PickleSerializerTests(SimpleTestCase):
+    @ignore_warnings(category=RemovedInDjango50Warning)
     def test_serializer_protocol(self):
         serializer = PickleSerializer(protocol=3)
         self.assertEqual(serializer.protocol, 3)
@@ -467,12 +469,21 @@ class PickleSerializerTests(SimpleTestCase):
         serializer = PickleSerializer()
         self.assertEqual(serializer.protocol, pickle.HIGHEST_PROTOCOL)
 
+    @ignore_warnings(category=RemovedInDjango50Warning)
     def test_serializer_loads_dumps(self):
         serializer = PickleSerializer()
         test_data = "test data"
         dump = serializer.dumps(test_data)
         self.assertEqual(serializer.loads(dump), test_data)
 
+    def test_serializer_warning(self):
+        msg = (
+            "PickleSerializer is deprecated due to its security risk. Use "
+            "JSONSerializer instead."
+        )
+        with self.assertRaisesMessage(RemovedInDjango50Warning, msg):
+            PickleSerializer()
+
 
 def register_tests(test_class, method_name, test_func, exclude=()):
     """
