@@ -37,6 +37,7 @@ from django.test import (
 )
 from django.urls import reverse_lazy
 from django.utils.decorators import async_only_middleware
+from django.views.generic import RedirectView
 
 from .views import TwoArgException, get_view, post_view, trace_view
 
@@ -221,6 +222,14 @@ class ClientTest(TestCase):
         response = self.client.get("/get_view/")
         self.assertEqual(response.resolver_match.url_name, "get_view")
 
+    def test_response_resolver_match_class_based_view(self):
+        """
+        The response ResolverMatch instance can be used to access the CBV view
+        class.
+        """
+        response = self.client.get("/accounts/")
+        self.assertIs(response.resolver_match.func.view_class, RedirectView)
+
     @modify_settings(MIDDLEWARE={"prepend": "test_client.tests.middleware_urlconf"})
     def test_response_resolver_match_middleware_urlconf(self):
         response = self.client.get("/middleware_urlconf_view/")
@@ -428,10 +437,10 @@ class ClientTest(TestCase):
         response = self.client.post("/form_view/", post_data)
         self.assertContains(response, "This field is required.", 3)
         self.assertTemplateUsed(response, "Invalid POST Template")
-
-        self.assertFormError(response, "form", "email", "This field is required.")
-        self.assertFormError(response, "form", "single", "This field is required.")
-        self.assertFormError(response, "form", "multi", "This field is required.")
+        form = response.context["form"]
+        self.assertFormError(form, "email", "This field is required.")
+        self.assertFormError(form, "single", "This field is required.")
+        self.assertFormError(form, "multi", "This field is required.")
 
     def test_form_error(self):
         "POST erroneous data to a form"
@@ -446,7 +455,9 @@ class ClientTest(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(response, "Invalid POST Template")
 
-        self.assertFormError(response, "form", "email", "Enter a valid email address.")
+        self.assertFormError(
+            response.context["form"], "email", "Enter a valid email address."
+        )
 
     def test_valid_form_with_template(self):
         "POST valid data to a form using multiple templates"
@@ -471,10 +482,10 @@ class ClientTest(TestCase):
         self.assertTemplateUsed(response, "form_view.html")
         self.assertTemplateUsed(response, "base.html")
         self.assertTemplateNotUsed(response, "Invalid POST Template")
-
-        self.assertFormError(response, "form", "email", "This field is required.")
-        self.assertFormError(response, "form", "single", "This field is required.")
-        self.assertFormError(response, "form", "multi", "This field is required.")
+        form = response.context["form"]
+        self.assertFormError(form, "email", "This field is required.")
+        self.assertFormError(form, "single", "This field is required.")
+        self.assertFormError(form, "multi", "This field is required.")
 
     def test_form_error_with_template(self):
         "POST erroneous data to a form using multiple templates"
@@ -491,7 +502,9 @@ class ClientTest(TestCase):
         self.assertTemplateUsed(response, "base.html")
         self.assertTemplateNotUsed(response, "Invalid POST Template")
 
-        self.assertFormError(response, "form", "email", "Enter a valid email address.")
+        self.assertFormError(
+            response.context["form"], "email", "Enter a valid email address."
+        )
 
     def test_unknown_page(self):
         "GET an invalid URL"
