@@ -12,6 +12,7 @@ from django.utils.http import (
     int_to_base36,
     is_same_domain,
     parse_etags,
+    parse_header_parameters,
     parse_http_date,
     quote_etag,
     url_has_allowed_host_and_scheme,
@@ -411,6 +412,13 @@ class HttpDateProcessingTests(unittest.TestCase):
             datetime(1994, 11, 6, 8, 49, 37, tzinfo=timezone.utc),
         )
 
+    def test_parsing_asctime_nonascii_digits(self):
+        """Non-ASCII unicode decimals raise an error."""
+        with self.assertRaises(ValueError):
+            parse_http_date("Sun Nov  6 08:49:37 １９９４")
+        with self.assertRaises(ValueError):
+            parse_http_date("Sun Nov １２ 08:49:37 1994")
+
     def test_parsing_year_less_than_70(self):
         parsed = parse_http_date("Sun Nov  6 08:49:37 0037")
         self.assertEqual(
@@ -428,3 +436,39 @@ class EscapeLeadingSlashesTests(unittest.TestCase):
         for url, expected in tests:
             with self.subTest(url=url):
                 self.assertEqual(escape_leading_slashes(url), expected)
+
+
+class ParseHeaderParameterTests(unittest.TestCase):
+    def test_basic(self):
+        tests = [
+            ("text/plain", ("text/plain", {})),
+            ("text/vnd.just.made.this.up ; ", ("text/vnd.just.made.this.up", {})),
+            ("text/plain;charset=us-ascii", ("text/plain", {"charset": "us-ascii"})),
+            (
+                'text/plain ; charset="us-ascii"',
+                ("text/plain", {"charset": "us-ascii"}),
+            ),
+            (
+                'text/plain ; charset="us-ascii"; another=opt',
+                ("text/plain", {"charset": "us-ascii", "another": "opt"}),
+            ),
+            (
+                'attachment; filename="silly.txt"',
+                ("attachment", {"filename": "silly.txt"}),
+            ),
+            (
+                'attachment; filename="strange;name"',
+                ("attachment", {"filename": "strange;name"}),
+            ),
+            (
+                'attachment; filename="strange;name";size=123;',
+                ("attachment", {"filename": "strange;name", "size": "123"}),
+            ),
+            (
+                'form-data; name="files"; filename="fo\\"o;bar"',
+                ("form-data", {"name": "files", "filename": 'fo"o;bar'}),
+            ),
+        ]
+        for header, expected in tests:
+            with self.subTest(header=header):
+                self.assertEqual(parse_header_parameters(header), expected)
